/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <signal.h>

#ifndef WIN32
#include <unistd.h>
#endif

#include <iostream>
#include <fstream>

#include <tools.h>
#include <folder.h>
#include <logf.h>
#include <stringtools.h>
#include <sbase64.h>
#ifndef DBOX_STANDALONE
#include <dboxversion.h>
#endif
#include "dboximap4.h"
#include "imap4fetch.h"
#include "imap4store.h"
#include "imap4copy.h"

void imap4alarm(int signal)
 {
  std::cout << "* BYE logout because of timeout\r\n";
  logf.printf("imap4alarm", "logout because of timeout");
  exit(0);
 }

void dboximap4t::doimap4()
 {
  signal(SIGALRM, imap4alarm);

  timeout_minutes=100;

  #ifdef DBOX_STANDALONE
   {
    char hostname[TEXTLEN]="";
    gethostname(hostname, TEXTLEN);
    std::cout << "* OK " << hostname << " DKIMAP4 IMAP Server\r\n";
   }
  #else
  std::cout << "* OK " << config.boxname << " DBOX IMAP Server (version "
            << DBOXVERSIONNR << ") ready.\r\n";
  #endif

  getpeer();

  logout=FALSE;
  last_status_update=0;
  last_index_file_size=0;
  largest_id=0;

  while(!logout)
   {
    if(get_and_parse_command())
     {
      channel_lost();
      return;
     }

    command_ok=TRUE;

    if(stringcasecmp(command, "CAPABILITY")==0)
     {
      std::cout << "* " << command
                << " IMAP4 IMAP4rev1 AUTH=LOGIN ACL NAMESPACE\r\n";
     }
    else if(stringcasecmp(command, "NOOP")==0)
     {
     }
    else if(stringcasecmp(command, "DK")==0)
     {
      for(unsigned argnr=0; argnr<arguments.size(); argnr++)
       {
        std::cout << "DK: \"" << arguments[argnr].data << "\" "
                  << "end_list: " << arguments[argnr].end_list << " "
                  << "begin_list: " << arguments[argnr].begin_list
                  << "\r\n";
       }
     }
    else if(stringcasecmp(command, "LOGOUT")==0)
     {
      std::cout << "* BYE as requested\r\n";

      if(transfer)
       {
        logf.printf("doimap4", "%s: logout by user request", up.name);
        #ifndef DBOX_STANDALONE
        logf.fileprintf(userlogfile, "doimap4", "logout by user request");
        #endif
       }
      else
        logf.printf("doimap4", "logout by user request");

      logout=TRUE;
     }
    else if(stringcasecmp(command, "AUTHENTICATE")==0 && !transfer)
     {
      if(arguments.size()==0)
       {
        std::cout << tag << " NO Sorry, AUTHENTICATE needs arguments\r\n";
        command_ok=FALSE;
       }
      else if(stringcasecmp(arguments[0].data, "LOGIN")==0)
       {
        std::cout << "+ VXNlciBOYW1lAA==\r\n";
        if(getcommand()) { channel_lost(); return; }
        up.setname(sbase64_decode(commandline));

        std::cout << "+ UGFzc3dvcmQA\r\n";
        if(getcommand()) { channel_lost(); return; }

        if(login(sbase64_decode(commandline)))
         {
          std::cout << tag << " NO invalid password\r\n";
          command_ok=FALSE;
         }
       }
      else
       {
        std::cout << tag << " NO Sorry, authentication method "
             << arguments[0].data << " is not supported "
                "up to now.\r\n";
        command_ok=FALSE;
       }
     }
    else if(stringcasecmp(command, "LOGIN")==0 && !transfer)
     {
      if(arguments.size()!=2)
       {
        std::cout << tag << " BAD invalid username\r\n";
        command_ok=FALSE;
       }
      else
       {
        up.setname(arguments[0].data);
        if(login(arguments[1].data))
         {
          std::cout << tag << " NO invalid password\r\n";
          command_ok=FALSE;
         }
       }
     }
    else if(transfer)
     {
      if(stringcasecmp(command, "SELECT")==0 ||
         stringcasecmp(command, "EXAMINE")==0)
        doselect();
      else if(stringcasecmp(command, "CREATE")==0)
        docreate();
      else if(stringcasecmp(command, "NAMESPACE")==0)
        donamespace();
      else if(stringcasecmp(command, "GETACL")==0)
        dogetacl();
      else if(stringcasecmp(command, "LISTRIGHTS")==0)
        dolistrights();
      else if(stringcasecmp(command, "MYRIGHTS")==0)
        domyrights();
      else if(stringcasecmp(command, "SETACL")==0)
        dosetacl();
      else if(stringcasecmp(command, "DELETEACL")==0)
        dodeleteacl();
      else if(stringcasecmp(command, "DELETE")==0)
        dodelete();
      else if(stringcasecmp(command, "RENAME")==0)
        dorename();
      else if(stringcasecmp(command, "SUBSCRIBE")==0)
        dosubscribe(TRUE);
      else if(stringcasecmp(command, "UNSUBSCRIBE")==0)
        dosubscribe(FALSE);
      else if(stringcasecmp(command, "LIST")==0 ||
              stringcasecmp(command, "LSUB")==0)
        dolsublist();
      else if(stringcasecmp(command, "APPEND")==0)
        doappend();
      else if(stringcasecmp(command, "CHECK")==0)
       {
       }
      else if(stringcasecmp(command, "CLOSE")==0)
       {
        if(mailbox=="")
         {
          std::cout << tag << " NO no mailbox selected\r\n";
          command_ok=FALSE;
         }
        else
         {
          // TODO: remove deleted messages
          close();
          mailbox="";
         }
       }
      else if(stringcasecmp(command, "EXPUNGE")==0 && mailbox!="")
       {
        doexpunge();
       }
      else if(stringcasecmp(command, "SEARCH")==0 && mailbox!="")
       {
        dosearch(FALSE);
       }
      else if(stringcasecmp(command, "FETCH")==0 && mailbox!="")
       {
        fetch_set_commandt set_command(*this);
        set_command.do_set_command(FALSE);
       }
      else if(stringcasecmp(command, "PARTIAL")==0)
       {
        std::cout << tag << " NO command is obsolete\r\n";
        command_ok=FALSE;
       }
      else if(stringcasecmp(command, "STORE")==0 && mailbox!="")
       {
        store_set_commandt set_command(*this);
        set_command.do_set_command(FALSE);
       }
      else if(stringcasecmp(command, "COPY")==0 && mailbox!="")
       {
        copy_set_commandt set_command(*this);
        set_command.do_set_command(FALSE);
       }
      else if(stringcasecmp(command, "STATUS")==0)
       {
        dostatus();
       }
      else if(stringcasecmp(command, "UID")==0 && mailbox!="")
       {
        upstring(arguments[0].data);

        if(stringcasecmp(arguments[0].data, "FETCH")==0)
         {
          fetch_set_commandt set_command(*this);
          set_command.do_set_command(TRUE);
         }
        else if(stringcasecmp(arguments[0].data, "COPY")==0)
         {
          copy_set_commandt set_command(*this);
          set_command.do_set_command(TRUE);
         }
        else if(stringcasecmp(arguments[0].data, "STORE")==0)
         {
          store_set_commandt set_command(*this);
          set_command.do_set_command(TRUE);
         }
        else if(stringcasecmp(arguments[0].data, "SEARCH")==0)
         {
          dosearch(TRUE);
         }
        else
         {
          std::cout << tag << " NO not implemented\r\n";
          command_ok=FALSE;
         }

        if(command_ok)
         {
          status_update();
          std::cout << tag << " OK " << command << " "
                    << arguments[0].data << " completed\r\n";
          command_ok=FALSE;
         }
       }
      else
       {
        std::cout << tag << " BAD command unknown\r\n";
        command_ok=FALSE;
       }
     }
    else
     {
      std::cout << tag << " BAD command unknown\r\n";
      command_ok=FALSE;
     }

    if(command_ok)
     {
      status_update();
      std::cout << tag << " OK " << command << " completed\r\n";
     }
 
    fflush(stdout);
   }
 }

