#ifndef DBOX_DBOXIMAP4_H

#define DBOX_DBOXIMAP4_H

#ifdef DBOX_STANDALONE
#include <pop.h>
#else
#include "../dboxpop3/pop.h"
#endif

#include <mimeparse.h>

#define DBOXIMAP4_FLAG_Answered 0x01
#define DBOXIMAP4_FLAG_Flagged  0x02
#define DBOXIMAP4_FLAG_Draft    0x04

#define MAXARGUMENTS 100
#define MAXARGLEN 1000
#define MAXFOLDER 1000

class dboximap4t:public dboxpopt
 {
 public:
  std::string command, tag, argument_string;
  bool command_ok, logout, readonly;
  dword uidvalidity;

  struct argumentt
   {
    std::string data;
    bool begin_list, end_list, literal;
    dword literal_size;
    
    argumentt() { begin_list=end_list=literal=FALSE; }
   };
   
  std::vector<argumentt> arguments;
  std::vector<bool> liststack;

  std::string mailbox;
  std::string folder_owner, subfolder;
  folder_grantst grants;
  void get_folder_grants();
  
  dword largest_id, messages;
  off_t last_index_file_size;
  time_t last_status_update;

  class folder_et
   {
   public:
    std::string name;
    bool subscribed;
    
    folder_et() { subscribed=FALSE; }
   };
   
  std::vector<folder_et> folders;

  void send_string(const char *string);
  void send_string(const std::string &string);
  
  bool process_mailbox_name(std::string &mailbox, std::string &folder_owner,
                            std::string &subfolder, std::string &folder_name);

  void print_date(time_t t);
  time_t parse_date(const std::string &date);
  void parse_date(const std::string &date, struct tm &tm);
  void channel_lost();
  void doimap4();
  void parse();
  bool get_and_parse_command();
  char *parse_getnext(char *ptr);
  void showstatus();
  void status_update();
  void parse_args();
  bool matchfolder(const std::string &mask, const std::string &folder);
  bool matchfolder(const char *mask, const char *folder);
  void get_folder_list();
  void get_subscribed();
  bool check_mailbox_name(const std::string &mailbox_name);

  void doexpunge();
  void docreate();
  void dodelete();
  void doselect();
  void dostatus();
  void dolsublist();
  void dosubscribe(bool subscribe);
  void doappend();
  void doappend2(folderpt &new_fp);
  void dorename();
  void dogetacl();
  void donamespace();
  void domyrights();
  void dolistrights();
  void dodeleteacl();
  void dosetacl();
  
  void begin_list();
  void list_item();
  void end_list();

  void dosearch(bool uid);
  bool dosearch_inset(const char *message_set, unsigned nr);
  bool dosearch_header_match(dword nr, const std::string &header, const std::string &pattern);
  bool dosearch_pattern_match(const std::string &text, const std::string &pattern);
  bool dosearch(dword nr, unsigned int &argnr);
  bool dosearch_text_match(dword nr, const std::string &pattern);
  bool dosearch_body_match(dword nr, const std::string &pattern);
  bool dosearch_body_match(mimestructt &mimestruct, const std::string &pattern,
                           unsigned recursion);
  bool dosearch_check_on(const time_t &date1, const time_t &date2);

  dboximap4t();
 };
 
#endif
