/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: COPY command

*/

#include <iostream>
#include <fstream>

#include <stringtools.h>
#include <sfolderrfc.h>
#include "imap4copy.h"

bool copy_set_commandt::start()
 {
  std::string new_mailbox_name=dboximap4.arguments[argnr].data, 
              new_folder_owner, new_subfolder, new_folder_name;

  if(dboximap4.process_mailbox_name(new_mailbox_name, new_folder_owner,
                                    new_subfolder, new_folder_name))
    return TRUE;

  folder_grantst new_grants;
  get_folder_grants(new_folder_owner, new_subfolder,
                    dboximap4.up, dboximap4.gp,
                    new_grants);

  if(!new_grants.insert)
   {
    std::cout << dboximap4.tag << " NO permission denied\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  new_fp.setname(new_folder_name);

  if(new_fp.lock())
   {
    std::cout << dboximap4.tag << " NO failed to lock folder\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE; 
   }

  return FALSE;
 }

bool copy_set_commandt::msg()
 {
  if(dboximap4.get_body(nr-1))
   {
    std::cout << "* " << nr << " ERROR: failed to get message\r\n";
    return FALSE;
   }

  if(dboximap4.rfcm.parse() || 
     dboximap4.rfcm.headerlen==0)
   {
    std::cout << "* " << nr << " ERROR: invalid message\r\n";
    return FALSE;
   }

  dword new_nr;
  if(spool_in_folder(dboximap4.rfcm, new_fp, FALSE, "", "", new_nr))
   {
    std::cout << dboximap4.tag << " NO internal error\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  folderindext new_fi;

  std::string filename=new_fp.getpath();
  filename+="index";
  std::fstream new_index(filename.c_str(), std::ios::in|std::ios::out|std::ios::binary);

  if(!new_index)
   {
    std::cout << dboximap4.tag << " NO internal error 1\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  new_index.seekp(-(signed)sizeof(new_fi), ios::end);
  std::streampos pos=new_index.tellp();
  if(!new_index.read((char *)&new_fi, sizeof(new_fi)))
   {
    std::cout << dboximap4.tag << " NO internal error 2 " << pos << "\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  new_fi.flags.imapflags=dboximap4.fi.flags.imapflags;
  new_fi.flags.deleted=dboximap4.fi.flags.deleted;
  new_fi.flags.seen=dboximap4.fi.flags.seen;
  new_fi.received=dboximap4.fi.received;

  new_index.seekp(pos, ios::beg);
  new_index.write((char *)&new_fi, sizeof(new_fi));

  if(copyuid!="") copyuid+=",";
  copyuid+=i2string(unsigned(new_fi.id));

  return FALSE;
 }

void copy_set_commandt::end()
 {
  // COPYUID: RFC2359
  dword uidvalidity=atol(new_fp|"imap.uidvalidity");
  if(uidvalidity==0) uidvalidity=1;

  new_fp.unlock();

  std::cout << dboximap4.tag << " OK [COPYUID " << uidvalidity << " " 
            << copyuid << "] COPY completed\r\n";

  dboximap4.command_ok=FALSE;
 }
