/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#ifndef WIN32
#include <unistd.h>
#endif

#include <iostream>
#include <strstream>

#include <stringtools.h>
#include "imap4fetch.h"

unsigned fetch_set_commandt::count_lines(const char *msg, size_t size)
 {
  unsigned lines=0;

  for(; size>0; size--, msg++)
    if(*msg=='\n') lines++;

  return lines;
 }

unsigned fetch_set_commandt::count_lines(mimestructt &mimestruct)
 {
  return count_lines(dboximap4.rfcm.data+
    mimestruct.offset+mimestruct.headerlen,
    mimestruct.len);
 }

void fetch_set_commandt::filter_header(int startargnr)
 {
  const char *id, *text, *start;
  std::ostrstream header;

  if(dboximap4.get_header(nr-1))
    return;

  start=dboximap4.rfcm.data;

  while(!dboximap4.rfcm.parseline(&start, &id, &text) && id!=NULL)
   {
    int filterargnr;
    std::string text;

    for(filterargnr=startargnr;
        !dboximap4.arguments[filterargnr].end_list;
        filterargnr++)
     {
      dboximap4.rfcm.copyheaderid(text, id);
      if(stringcasecmp(text, dboximap4.arguments[filterargnr].data)==0)
       {
        header.write(id, start-id);        
        break;
       }
     }
   }

  header << "\r\n";

  std::cout << '{' << header.tellp() << "}\r\n";

   {
    char *str=header.str();
    std::cout.write(str, header.tellp());
    delete str;
   }
 }

void fetch_set_commandt::fetch_body_fields(mimestructt &mimestruct)
 {
  unsigned i;

  dboximap4.list_item(); // body_fld_param
  dboximap4.begin_list();
  for(i=0; i<mimestruct.content_type.attributes.size(); i++)
   {
    dboximap4.list_item();
    dboximap4.send_string(mimestruct.content_type.attributes[i].name);
    dboximap4.list_item();
    dboximap4.send_string(mimestruct.content_type.attributes[i].value);
   }
  dboximap4.end_list();

  dboximap4.list_item(); // body_fld_id
  std::cout << "NIL";

  dboximap4.list_item(); // body_fld_desc
  std::cout << "NIL";

  dboximap4.list_item(); // body_fld_enc
  if(mimestruct.content_transfer_encoding.value!="")
    dboximap4.send_string(mimestruct.content_transfer_encoding.value);
  else
    dboximap4.send_string("7BIT");

  dboximap4.list_item(); // body_fld_octets
  std::cout << mimestruct.len;
 }

void fetch_set_commandt::fetch_bodystructure(mimestructt &mimestruct)
 {
  dboximap4.begin_list();

  if(stringcasecmp(mimestruct.content_type.value, "multipart")==0) // mpart
   {
    if(mimestruct.sub.size()==0)
     {
      dboximap4.list_item();
      std::cout << "NIL";
     }
    else
     {
      for(unsigned i=0; i<mimestruct.sub.size(); i++)
       {
        dboximap4.list_item();
        fetch_bodystructure(mimestruct.sub[i]);
       }

      dboximap4.list_item();
      dboximap4.send_string(mimestruct.content_type.subvalue);
     }
   }
  else
   {
    dboximap4.list_item();

    dboximap4.send_string(mimestruct.content_type.value);

    if(stringcasecmp(mimestruct.content_type.value, "text")==0) // basic
     {
      dboximap4.list_item(); dboximap4.send_string(mimestruct.content_type.subvalue);
      fetch_body_fields(mimestruct);
      dboximap4.list_item(); std::cout << count_lines(mimestruct);
     }
    else if(stringcasecmp(mimestruct.content_type.value, "message")==0 &&
            stringcasecmp(mimestruct.content_type.subvalue, "rfc822")==0) // msg
     {
      rfcmessaget subpartm;

      subpartm.data=dboximap4.rfcm.data+mimestruct.offset+mimestruct.headerlen;
      subpartm.endofmail=subpartm.endoffile=subpartm.data+mimestruct.len;

      mimestructt substruct;
      substruct.offset=mimestruct.offset+mimestruct.headerlen;
      substruct.mimeparse(subpartm);

      dboximap4.list_item(); dboximap4.send_string(mimestruct.content_type.subvalue);
      fetch_body_fields(mimestruct);
      dboximap4.list_item(); fetch_envelope(subpartm);
      dboximap4.list_item(); fetch_bodystructure(substruct);
      dboximap4.list_item(); std::cout << count_lines(mimestruct);
     }
    else // basic
     {
      dboximap4.list_item();
      dboximap4.send_string(mimestruct.content_type.subvalue);
      fetch_body_fields(mimestruct);
     }
   }
  
  dboximap4.end_list();
 }

void fetch_set_commandt::fetch_bodystructure(const std::string &what)
 {
  if(dboximap4.get_structure(nr-1))
    return;

  dboximap4.list_item();
  std::cout << what;
  dboximap4.list_item();
  fetch_bodystructure(*dboximap4.mimestruct);
 }

void fetch_set_commandt::fetch_body_parse(const std::string &what,
  bool &peek, std::string &specification, std::string &part,
  bool &partial, dword &partial_start, dword &partial_size)
 {
  partial=peek=FALSE;
  const char *what_ptr=what.c_str();

  partial_start=0;
  partial_size=(dword)-1;

  if(strnicmp(what_ptr, "BODY", 4)==0)
    what_ptr+=4;

  if(strnicmp(what_ptr, ".PEEK", 5)==0)
   { what_ptr+=5; peek=TRUE; }

  if(what_ptr[0]=='[')
   {
    what_ptr++;

    char _part[MAXARGLEN];
    strmaxcpy(_part, what_ptr, MAXARGLEN-1);
    char *tptr=_part;

    while(isdigit(tptr[0]) || tptr[0]=='.') tptr++;

    char _specification[MAXARGLEN];
    strmaxcpy(_specification, tptr, MAXARGLEN-1);
    *tptr=0;

    tptr=strchr(_specification, ']');
    if(tptr!=NULL) *(tptr++)=0;

    part=_part;
    specification=_specification;
   }

  // TODO: case header.fields: next argument

  what_ptr=strchr(what.c_str(), '<');
  if(what_ptr!=NULL)
   {
    partial=TRUE;
    what_ptr++;
    partial_start=atol(what_ptr);
    while(isdigit(*what_ptr)) what_ptr++;
    if(*what_ptr=='.') 
     {
      what_ptr++;
      partial_size=atol(what_ptr);
     }
   } 
 }

void fetch_set_commandt::fetch_body4(const char *ptr,
  dword data_size, bool partial,
  dword partial_start, dword partial_size)
 {
  #if 0
  // Netscape Bug Workaround
  if(partial && partial_start==0)
    partial=FALSE;
  #endif

  if(partial)
    std::cout << "<" << partial_start << ">";

  dboximap4.list_item();

  if(partial)
   {
    if(partial_start>=data_size)
      data_size=0;
    else
     {
      data_size-=partial_start;
      if(data_size>partial_size) data_size=partial_size;
     }
   }
  else
    partial_start=0;

  std::cout << '{' << data_size << "}\r\n";

  if(data_size!=0)
    std::cout.write(ptr+partial_start, data_size);
 }

void fetch_set_commandt::fetch_body3(
  const std::string &specification,
  bool partial, dword partial_start, dword partial_size,
  const std::string &display_response)
 {
  if(specification=="")
   {
    if(!dboximap4.get_body(nr-1))
     {
      dboximap4.list_item();
      std::cout << display_response;

      fetch_body4(dboximap4.rfcm.data,
                  dboximap4.rfcm.headerlen+dboximap4.rfcm.len,
                  partial, partial_start, partial_size);
     }
   }
  else if(stricmp(specification.c_str(), "HEADER")==0)
   {
    if(!dboximap4.get_header(nr-1))
     {
      dboximap4.list_item();
      std::cout << display_response;
      fetch_body4(dboximap4.rfcm.data,
                  dboximap4.rfcm.headerlen,
                  partial, partial_start, partial_size);
     }
   }
  else if(stricmp(specification.c_str(), "TEXT")==0)
   {
    if(!dboximap4.get_body(nr-1))
     {
      dboximap4.list_item();
      std::cout << display_response;
      fetch_body4(dboximap4.rfcm.data+dboximap4.rfcm.headerlen,
                  dboximap4.rfcm.len,
                  partial, partial_start, partial_size);
     }
   }
  else if(strnicmp(specification.c_str(), "HEADER.FIELDS", 13)==0)
   {
    if(!dboximap4.get_header(nr-1))
     {
      bool inverse=FALSE;
      int startargnr;

      if(stricmp(specification.c_str()+13, ".NOT")==0)
        inverse=TRUE;

      argnr++;
      if(!dboximap4.arguments[argnr].begin_list) // parse error
        return;

      argnr++;
      startargnr=argnr;

      dboximap4.list_item();
      std::cout << "BODY[" << specification;
      dboximap4.list_item();
      dboximap4.begin_list();

      while(argnr<dboximap4.arguments.size() && 
            !dboximap4.arguments[argnr].end_list)
       {
        dboximap4.list_item();
        //upstring(arguments[argnr].data); // Hm?
        dboximap4.send_string(dboximap4.arguments[argnr].data);
        argnr++;
       }

      dboximap4.end_list();

      std::cout << ']';
      if(dboximap4.arguments[argnr].data=="]") argnr++;

      dboximap4.list_item();

      filter_header(startargnr);
     }
   }
 } 

void fetch_set_commandt::fetch_body3_part2(
  mimestructt &mimestruct,
  const std::string &specification,
  bool partial, dword partial_start, dword partial_size,
  const std::string &display_response)
 {
  if(specification=="")
   {
    dboximap4.list_item();
    std::cout << display_response;
    fetch_body4(dboximap4.rfcm.data+
                mimestruct.offset+mimestruct.headerlen,
                mimestruct.len,
                partial, partial_start, partial_size);
   }
  else if(stricmp(specification.c_str(), "MIME")==0)
   {
    dboximap4.list_item();
    std::cout << display_response;
    fetch_body4(dboximap4.rfcm.data+
                mimestruct.offset, 
                mimestruct.headerlen,
                partial, partial_start, partial_size);
   }
  else
   {
    if(mimestruct.sub.size()==1)
     {
      struct mimestructt &_mimestruct=mimestruct.sub[0];

      if(stricmp(specification.c_str(), "HEADER")==0)
       {
        dboximap4.list_item();
        std::cout << display_response;
        fetch_body4(dboximap4.rfcm.data+
                    _mimestruct.offset, 
                    _mimestruct.headerlen,
                    partial, partial_start, partial_size);
       }
      else if(stricmp(specification.c_str(), "TEXT")==0)
       {
        dboximap4.list_item();
        std::cout << display_response;
        fetch_body4(dboximap4.rfcm.data+
                    _mimestruct.offset+mimestruct.headerlen,
                    _mimestruct.len,
                    partial, partial_start, partial_size);
       }
      else if(strnicmp(specification.c_str(), "HEADER.FIELDS", 13)==0)
       { // TODO
        bool inverse=FALSE;
        int startargnr;

        if(stricmp(specification.c_str()+13, ".NOT")==0)
          inverse=TRUE;

        argnr++;
        if(!dboximap4.arguments[argnr].begin_list) // parse error
          return;

        argnr++;
        startargnr=argnr;

        dboximap4.list_item();
        std::cout << "BODY[" << specification;
        dboximap4.list_item();
        dboximap4.begin_list();

        while(argnr<dboximap4.arguments.size() && 
              !dboximap4.arguments[argnr].end_list)
         {
          dboximap4.list_item();
          //upstring(arguments[argnr].data); // Hm?
          dboximap4.send_string(dboximap4.arguments[argnr].data);
          argnr++;
         }

        dboximap4.end_list();

        std::cout << ']';
        if(dboximap4.arguments[argnr].data=="]") argnr++;

        dboximap4.list_item();

        filter_header(startargnr);
       }
     }
   }
 }

void fetch_set_commandt::fetch_body3_part(
  mimestructt &mimestruct,
  const std::string &specification, const std::string &part,
  bool partial, dword partial_start, dword partial_size,
  const std::string &display_response)
 {
  if(part=="1" && mimestruct.sub.size()==0 &&
     specification=="")
   {
    fetch_body3("TEXT", partial,
      partial_start, partial_size,
      display_response);
   }
  else
   {
    unsigned sub=atoi(part.c_str());

    if(!isdigit(part[0]) ||
       sub>mimestruct.sub.size() || 
       sub==0)
     {
      dboximap4.list_item(); 
      std::cout << display_response;
      dboximap4.list_item();
      std::cout << "NIL";
     }
    else
     {
      const char *tptr=part.c_str();
      while(isdigit(*tptr)) tptr++;
      if((*tptr)=='.') tptr++;

      if(tptr[0]==0)
       {
        fetch_body3_part2(mimestruct.sub[sub-1],
          specification,
          partial, partial_start, partial_size,
          display_response);
       }
      else
       {
        fetch_body3_part(mimestruct.sub[sub-1], specification,
          tptr, partial, partial_start, partial_size, display_response);
       }
     }
   }
 } 

void fetch_set_commandt::fetch_body2(const std::string &what,
                                     const std::string &response)
 {
  bool peek, partial;
  std::string specification, part;
  dword partial_start, partial_size;

  fetch_body_parse(what, peek, specification, part,
                   partial, partial_start, partial_size);

  std::string display_response;
  if(response=="")
    display_response="BODY["+part+specification+"]";
  else
    display_response=response;

  if(part=="")
   {
    fetch_body3(specification, partial,
      partial_start, partial_size, display_response);
   }
  else
   {
    if(dboximap4.get_structure(nr-1))
      return;

    fetch_body3_part(*dboximap4.mimestruct,
      specification, part, partial,
      partial_start, partial_size, display_response);
   }

  if(!peek)
   {
    dboximap4.get_folderindex(nr-1);
    if(!dboximap4.fi.flags.seen)
     {
      if(!dboximap4.fp.lock())
       {
        lseek(dboximap4.index, (nr-1)*sizeof dboximap4.fi, SEEK_SET);
        read(dboximap4.index, &dboximap4.fi, sizeof dboximap4.fi);
        dboximap4.fi.flags.seen=TRUE;
        lseek(dboximap4.index, (nr-1)*sizeof dboximap4.fi, SEEK_SET);
        write(dboximap4.index, &dboximap4.fi, sizeof dboximap4.fi);
        dboximap4.fp.unlock();
       }

      fetch("FLAGS");
     }
   }
 }

//
// send a string to the client
//
// if the string is clean, send quoted
// if not, send as literal
//

static void trim(char *string)
 {
  char *tptr, *tptr2;

  tptr=string;
  while(isspace(*tptr)) tptr++;

  if(tptr!=string)
   {
    tptr2=string;
    while(*tptr!=0) *(tptr2++)=*(tptr++);
    *tptr2=0;
   }

  tptr=string;
  while(*tptr!=0) tptr++;
  while(tptr>string && isspace(tptr[-1]))
    *(--tptr)=0;
 }

void fetch_set_commandt::address_to_imap(std::ostrstream &realname, std::ostrstream &email)
 {
  dboximap4.begin_list();

  realname << '\x0';
  email << '\x0';

   {
    char *str=realname.str();
    if(str==NULL) str=strdup("");
    trim(str);
    dboximap4.list_item(); dboximap4.send_string(str);
    delete str;
   }

  dboximap4.list_item(); std::cout << "NIL";

   {
    char *str=email.str();
    if(str==NULL) str=strdup("");
    trim(str);
    
    char *tptr=strchr(str, '@');
    if(tptr!=NULL) *(tptr++)=0;
    dboximap4.list_item(); dboximap4.send_string(str);
    dboximap4.list_item(); dboximap4.send_string(tptr==NULL?"":tptr);
    delete str;
   }

  dboximap4.end_list();
 }

void fetch_set_commandt::address_to_imap(const char *&address)
 {
  int state=0;
  bool done=FALSE;
  std::ostrstream text1, text2;

  while(!done)
   {
    if(address[0]=='"')
     {
      address++;
      while(address[0]!='"' && address[0]!=0)
        text1 << (address++)[0];
     }
    else if(state==0)
     {
      switch(address[0])
       {
       case '<':  // address has form realname <user@host>
        state=1;
        break;

       case '(':  // address has form user@host (realname)
        state=2;
        break;

       case ',':  // address has form user@host
       case 0:
        address_to_imap(text2, text1);
        done=TRUE;
        break;

       default:
        text1 << address[0];
       }
     }
    else if(state==1) // address has form realname <user@host>
     {
      if(address[0]=='>' || address[0]==0)
       {
        address_to_imap(text1, text2);
        done=TRUE;
       }
      else
        text2 << address[0];
     }
    else if(state==2) // address has form user@host (realname)
     {
      if(address[0]==')' || address[0]==0)
       {
        address_to_imap(text2, text1);
        done=TRUE;
       }
      else 
        text2 << address[0];
     }

    if(address[0]!=0) address++;
   }

  while(isspace(address[0]) || address[0]==',') address++;
 }

void fetch_set_commandt::address_list_to_imap(const std::string &address)
 {
  const char *tptr=address.c_str();

  dboximap4.begin_list();
  while(tptr[0]!=0)
   {
    dboximap4.list_item();
    address_to_imap(tptr);
   }
  dboximap4.end_list();
 }

void fetch_set_commandt::send_header(rfcmessaget &rfcm, const std::string &name)
 {
  std::string header; 

  if(rfcm.findsingleheader(header, name))
   {
    if(stringcasecmp(name, "reply-to")==0 || // default those fields
       stringcasecmp(name, "sender")==0)
      send_header(rfcm, "from");
    else
      std::cout << "NIL";
   }
  else
   {
    if(stringcasecmp(name, "from")==0 ||
       stringcasecmp(name, "sender")==0 ||
       stringcasecmp(name, "reply-to")==0 || 
       stringcasecmp(name, "to")==0 ||
       stringcasecmp(name, "cc")==0 ||
       stringcasecmp(name, "bcc")==0)
      address_list_to_imap(header);
    else
      dboximap4.send_string(header);
   }
 }

void fetch_set_commandt::fetch_envelope(rfcmessaget &rfcm)
 {
  dboximap4.begin_list();
  dboximap4.list_item(); send_header(rfcm, "date");
  dboximap4.list_item(); send_header(rfcm, "subject");
  dboximap4.list_item(); send_header(rfcm, "from");
  dboximap4.list_item(); send_header(rfcm, "sender");
  dboximap4.list_item(); send_header(rfcm, "reply-to");
  dboximap4.list_item(); send_header(rfcm, "to");
  dboximap4.list_item(); send_header(rfcm, "cc");
  dboximap4.list_item(); send_header(rfcm, "bcc");
  dboximap4.list_item(); send_header(rfcm, "in-reply-to");
  dboximap4.list_item(); send_header(rfcm, "message-id");
  dboximap4.end_list();
 }

void fetch_set_commandt::fetch_body(const std::string &what)
 {
  if(stringcasecmp(what, "ENVELOPE")==0)
   {
    if(!dboximap4.get_header(nr-1))
     {
      dboximap4.list_item();
      std::cout << what;
      dboximap4.list_item();
      fetch_envelope(dboximap4.rfcm);
     }
   }
  else if(stringcasecmp(what, "BODY")==0 ||
          stringcasecmp(what, "BODYSTRUCTURE")==0)
    fetch_bodystructure(what);
  else if(strnicmp(what.c_str(), "BODY", 4)==0)
    fetch_body2(what, "");
  else if(stringcasecmp(what, "RFC822")==0)
    fetch_body2("BODY[]", what);
  else if(stringcasecmp(what, "RFC822.TEXT")==0)
    fetch_body2("BODY[TEXT]", what);
  else if(stringcasecmp(what, "RFC822.HEADER")==0)
    fetch_body2("BODY.PEEK[HEADER]", what);
  else if(stringcasecmp(what, "RFC822.SIZE")==0)
   {
    if(!dboximap4.get_header(nr-1))
     {
      dboximap4.list_item();
      std::cout << what;
      dboximap4.list_item();
      std::cout << (dboximap4.rfcm.headerlen+dboximap4.fi.len);
     }
   }
 }

void fetch_set_commandt::fetch(const std::string &what)
 {
  if(stringcasecmp(what, "ALL")==0)
   {
    fetch("FLAGS");
    fetch("INTERNALDATE");
    fetch("RFC822.SIZE");
    fetch("ENVELOPE");
   }
  else if(strnicmp(what.c_str(), "BODY", 4)==0 ||
          strnicmp(what.c_str(), "RFC822", 6)==0 ||
          stringcasecmp(what, "ENVELOPE")==0 ||
          stringcasecmp(what, "BODYSTRUCTURE")==0)
   {
    fetch_body(what);
   }
  else if(stringcasecmp(what, "FAST")==0)
   {
    fetch("FLAGS");
    fetch("INTERNALDATE");
    fetch("RFC822.SIZE");
   }
  else if(stringcasecmp(what, "FLAGS")==0)
   {
    dboximap4.list_item();
    std::cout << "FLAGS";
    dboximap4.list_item();
    dboximap4.begin_list();
    if(dboximap4.fi.flags.imapflags&DBOXIMAP4_FLAG_Answered)
     { dboximap4.list_item(); std::cout << "\\Answered"; }
    if(dboximap4.fi.flags.imapflags&DBOXIMAP4_FLAG_Flagged)
     { dboximap4.list_item(); std::cout << "\\Flagged"; }
    if(dboximap4.fi.flags.deleted) 
     { dboximap4.list_item(); std::cout << "\\Deleted"; }
    if(dboximap4.fi.flags.seen)
     { dboximap4.list_item(); std::cout << "\\Seen"; }
    if(dboximap4.fi.flags.imapflags&DBOXIMAP4_FLAG_Draft)
     { dboximap4.list_item(); std::cout << "\\Draft"; }
    if(FALSE)
     { dboximap4.list_item(); std::cout << "\\Recent"; }
    dboximap4.end_list();
   }
  else if(stringcasecmp(what, "FULL")==0)
   {
    fetch("FLAGS");
    fetch("INTERNALDATE");
    fetch("RFC822.SIZE");
    fetch("ENVELOPE");
    fetch("BODY");
   }
  else if(stringcasecmp(what, "INTERNALDATE")==0)
   {
    dboximap4.list_item();
    std::cout << "INTERNALDATE";
    dboximap4.list_item();
    dboximap4.print_date(dboximap4.fi.received);
   }
  else if(stringcasecmp(what, "UID")==0)
   {
    dboximap4.list_item();
    std::cout << "UID";
    dboximap4.list_item();
    std::cout << dboximap4.fi.id;
   }
 }

bool fetch_set_commandt::msg()
 {
  std::string item;
  bool uid_already_done=FALSE;

  dboximap4.get_folderindex(nr-1);

  argnr=uid?2:1;

  std::cout << "* " << nr << " " 
            << (uid?dboximap4.arguments[0].data:dboximap4.command)
            << " ";

  dboximap4.begin_list();

  for(; argnr<dboximap4.arguments.size(); argnr++)
   {
    if(dboximap4.arguments[argnr].begin_list ||
       dboximap4.arguments[argnr].end_list) 
     {
     }
    else
     {
      item=dboximap4.arguments[argnr].data;
      upstring(item);
      fetch(item);
      if(stringcasecmp(item, "UID")==0) uid_already_done=TRUE;
     }
   }  

  if(uid && !uid_already_done)
    fetch("UID");   

  dboximap4.end_list();

  std::cout << "\r\n";

  return FALSE;
 }

