/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <sys/stat.h>

#ifndef WIN32
#include <unistd.h>
#endif

#include <strstream>
#include <iostream>

#include <stringtools.h>
#include <tools.h>
#include <logf.h>
#include <rfc1123date.h>
#include "dboximap4.h"
#include "mod-utf7.h"

void dboximap4t::channel_lost()
 {
  if(transfer)
   {
    logf.printf("doimap4", "%s: channel lost", up.name);
    #ifndef DBOX_STANDALONE
    logf.fileprintf(userlogfile, "doimap4", "channel lost");
    #endif
   }
  else
    logf.printf("doimap4", "channel lost");
 }

bool dboximap4t::check_mailbox_name(const std::string &mailbox_name)
 {
  for(unsigned i=0; i<mailbox_name.size(); i++)
   {
    register char ch=mailbox_name[i];

    if(isascii(ch) && iscntrl(ch))
      return TRUE;
   }

  return FALSE;
 }

void dboximap4t::get_folder_grants()
 {
  ::get_folder_grants(folder_owner, subfolder, up, gp, grants);
 }

void dboximap4t::send_string(const char *string)
 {
  const char *tptr;

  if(string[0]==0)
   {
    std::cout << "NIL";
    return;
   }

  for(tptr=string; *tptr!=0; tptr++)
    if(*tptr=='\n' || *tptr=='\r')
     {
      std::cout << '{' << strlen(string) << "}\r\n" << string;
      return;
     }

  std::cout << '"';

  for(tptr=string; *tptr!=0; tptr++)
   {
    if(*tptr=='\\' || *tptr=='"') std::cout << '\\';
    std::cout << char(*tptr);
   }

  std::cout << '"';
 }

void dboximap4t::send_string(const std::string &string)
 {
  send_string(string.c_str());
 }

bool dboximap4t::process_mailbox_name(std::string &mailbox_name,
                                      std::string &folder_owner,
                                      std::string &subfolder,
                                      std::string &folder_name)
 {
  mailbox_name=decode_mod_utf7(mailbox_name);

  if(stringcasecmp(mailbox_name, "INBOX")==0)
    mailbox_name="INBOX";

  folder_owner=up.name;

  if(get_subfolder(folder_owner, mailbox_name, subfolder))
   {
    std::cout << tag << " NO no such mailbox\r\n";
    command_ok=FALSE;
    return TRUE;
   }
 
  if(mailfolderalias(folder_owner, subfolder))
   {
    std::cout << tag << " NO alias no longer valid\r\n";
    command_ok=FALSE;
    return TRUE;
   }

  folder_name=subfolder+"@"+folder_owner;

  return FALSE;
 }

#define PARSE_DATE_SKIP(p,c) while(*(p)!=(c) && *(p)!=0) (p)++; \
                             if(*(p)==0) return; else (p)++

void dboximap4t::parse_date(const std::string &date, struct tm &tm)
 {
  const char *ptr=date.c_str();

  if(ptr[0]==0) return;
  tm.tm_mday=atoi(ptr);
  PARSE_DATE_SKIP(ptr, '-');

  unsigned mon=0;
  for(mon=0; mon<12; mon++)
    if(strnicmp(rfc_months[mon], ptr, 3)==0)
      break;

  if(mon==12) return;
  tm.tm_mon=mon;

  PARSE_DATE_SKIP(ptr, '-');

  tm.tm_year=atoi(ptr)-1900;
  PARSE_DATE_SKIP(ptr, ' ');

  tm.tm_hour=atoi(ptr);
  PARSE_DATE_SKIP(ptr, ':');
    
  tm.tm_min=atoi(ptr);
  PARSE_DATE_SKIP(ptr, ':');
    
  tm.tm_sec=atoi(ptr);
 }

time_t dboximap4t::parse_date(const std::string &date)
 {
  struct tm tm;

  time_t t;
  time(&t);

  tm=*localtime(&t);

  parse_date(date, tm);
  
  return mktime(&tm);
 }

void dboximap4t::print_date(time_t t)
 {
  struct tm *tm;

  tm=localtime(&t);

  signed long int tzdiff;

  #ifdef __CYCWIN32__
  tzdiff=0;
  #else
  tzdiff=-tm->tm_gmtoff;
  //if(tm->tm_isdst>0) tzdiff+=60*60;
  #endif

  printf("\"%2d-%s-%04d %02d:%02d:%02d %c%02ld%02ld\"",
    tm->tm_mday, rfc_months[tm->tm_mon], tm->tm_year+1900,
    tm->tm_hour, tm->tm_min, tm->tm_sec, 
    tzdiff<=0?'+':'-',
    (labs(tzdiff)/60/60)%100, (labs(tzdiff)/60)%60);
 }

void dboximap4t::begin_list()
 {
  liststack.push_back(TRUE);
  std::cout << '(';
 }

void dboximap4t::end_list()
 {
  if(liststack.size()==0) abort();
  std::cout << ')';
  liststack.pop_back();
 }

void dboximap4t::list_item()
 {
  if(liststack.size()==0) abort();

  if(liststack.back())
    liststack.back()=FALSE;
  else
    std::cout << ' ';
 }

dboximap4t::dboximap4t():dboxpopt()
 {
  command_ok=logout=readonly=FALSE; 
  protocol="imap4";
  mailbox="";
  messages=0;
  uidvalidity=1;
 }

void dboximap4t::showstatus()
 {
  folderindext fi;
  dword exists=0, recent=0;
  largest_id=0;
  messages=0;

  if(mailbox!="" && index>=0 && header>=0 && !logout)
   {
    lseek(index, 0, SEEK_SET);

    while(read(index, &fi, sizeof fi)==sizeof fi)
     {
      exists++;
      messages++;
      if(fi.id>largest_id) largest_id=fi.id;
     }

    std::cout << "* " << exists << " EXISTS\r\n";
    std::cout << "* " << recent << " RECENT\r\n";
   }
 }

void dboximap4t::status_update()
 {
  time_t t;

  if(!transfer || mailbox=="") return;

  time(&t);

  #ifdef DBOX_STANDALONE
  if(mailbox=="INBOX")
    movemail();
  #endif

  // if(t>(last_status_update+5))
   {
    struct stat statbuf;
    std::string filename=std::string(fp.getpath())+"index";

    if(stat(filename.c_str(), &statbuf)==0 && 
       statbuf.st_size!=last_index_file_size)
     {
      open_files();
      showstatus();
      last_index_file_size=statbuf.st_size;
      last_status_update=t;
     }
   }

  if(t>(last_status_update+60))
   {
    showstatus();
    last_status_update=t;
   }
 }

void dboximap4t::parse_args()
 {
  const char *tptr=argument_string.c_str();

  while(*tptr!=0)
   {
    argumentt argument;

    switch(*tptr)
     {
     case '(':
      tptr++;
      argument.begin_list=TRUE;
      break;

     case ')':
      tptr++;
      argument.end_list=TRUE;
      break;

     case '"': // quoted
      tptr++;
       {
        bool quoted=FALSE;
        while(tptr[0]!=0 && (tptr[0]!='"' && !quoted))
         {
          if(!quoted && tptr[0]=='\\')
            quoted=TRUE;
          else
           {
            quoted=FALSE;
            argument.data+=tptr[0];
           }
          tptr++;
         }
       }

      if(tptr[0]=='"') tptr++;
      break;

     case '{': // literal
      tptr++;

      argument.literal=TRUE;
      argument.literal_size=atol(tptr);

      while(tptr[0]!=0 && tptr[0]!='}')
        tptr++;

      if(tptr[0]=='}') tptr++;
      break;

     default: // "normal"
      while(tptr[0]!=0 && tptr[0]!=' ' && tptr[0]!=')')
        argument.data+=*(tptr++);
     }

    if(*tptr==' ') tptr++;
    arguments.push_back(argument);
   }
 }

void dboximap4t::parse()
 {
  const char *tptr=commandline.c_str();

  tag="";
  while(*tptr!=0 && *tptr!=' ')
    tag+=*(tptr++);

  if(*tptr==' ') tptr++;

  command="";
  while(*tptr!=0 && *tptr!=' ')
    command+=*(tptr++);

  if(*tptr==' ') tptr++;

  argument_string=tptr;

  upstring(command);

  arguments.clear();
  parse_args();
 }

bool dboximap4t::get_and_parse_command()
 {
  unsigned args;

  if(getcommand()) return TRUE;

  #ifdef DEBUG
  #ifndef DBOX_STANDALONE
  logf.printf("debug", "%d >%s<", getpid(), commandline.c_str());
  #endif
  #endif

  parse();

  args=arguments.size();

  if(args==0) return FALSE;
  
  while(arguments[args-1].literal)
   {
    std::cout << "+ send literal\r\n";

     {
      std::ostrstream data;
      if(filecopy(cin, data, arguments[args-1].literal_size))
        return TRUE;

      arguments[args-1].data.assign(data.str(), 
        arguments[args-1].literal_size);
      data.freeze(FALSE);
     }    

    if(getcommand()) return TRUE;

    #ifdef DEBUG
    #ifndef DBOX_STANDALONE
    logf.printf("debug", ">%s<", commandline.c_str());
    #endif
    #endif

    if(commandline=="") return FALSE;
    if(commandline[0]!=' ') return FALSE;

    argument_string=(commandline.c_str()+1); // skip blank

    if(argument_string!="")   
      parse_args();

    if(args==arguments.size()) // nothing new
      return FALSE;

    args=arguments.size();
   }

  return FALSE;
 }
