/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <time.h>

#include <strstream>
#include <iostream>

#include <general.h>
#include <stringtools.h>
#include <mimedecode.h>
#include <rfc1522.h>
#include "dboximap4.h"

bool dboximap4t::dosearch_check_on(const time_t &date1, const time_t &date2)
 {
  struct tm tm1, tm2;

  tm1=*localtime(&date1);
  tm2=*localtime(&date2);

  return (tm1.tm_mday==tm2.tm_mday && 
          tm1.tm_mon==tm2.tm_mon &&
          tm1.tm_year==tm2.tm_year);
 }

bool dboximap4t::dosearch_inset(const char *message_set, unsigned nr)
 {
  dword number=0, nr1=0, nr2=0;
  bool sequence=FALSE;
  dword max_value=0xffffffff;

  while(message_set[0]!=0)
   {
    if(isdigit(message_set[0]))
     {
      number=atol(message_set);
      while(isdigit(message_set[0])) message_set++;
     }
    else if(message_set[0]=='*')
     {
      number=max_value;
      message_set++;
     }

    if(message_set[0]==':')
     {
      nr1=number;
      sequence=TRUE;
     }
    else if(sequence)
     {
      nr2=number;
      sequence=FALSE;

      if(nr2>max_value) nr2=max_value;

      if(nr>=nr1 && nr<=nr2) return TRUE;
     }
    else if(nr==number)
      return TRUE;

    if(message_set[0]!=0)
      message_set++;
   }

  return FALSE;
 }

bool dboximap4t::dosearch_pattern_match
  (const std::string &text, const std::string &pattern)
 {
  for(unsigned i=0; i<text.size(); i++)
    if(strnicmp(text.c_str()+i, pattern.c_str(), pattern.size())==0)
      return TRUE;

  return FALSE;
 }

bool dboximap4t::dosearch_header_match
  (dword nr, const std::string &header, const std::string &pattern)
 {
  get_header(nr-1);
  std::string text;
  rfcm.findsingleheader(text, header);
  return dosearch_pattern_match(decode_rfc1522(text), pattern);
 }

#include <logf.h>

bool dboximap4t::dosearch_body_match
  (mimestructt &mimestruct, const std::string &pattern, unsigned recursion)
 {
  if(mimestruct.content_type.value=="" ||
     (stringcasecmp(mimestruct.content_type.value, "text")==0 &&
     (stringcasecmp(mimestruct.content_type.subvalue, "plain")==0 ||
      stringcasecmp(mimestruct.content_type.subvalue, "x-vcard")==0 ||
      stringcasecmp(mimestruct.content_type.subvalue, "html")==0)))
   {
    std::string text=mime_decode(mimestruct, rfcm);
    return dosearch_pattern_match(text, pattern);
   }
  else if(stringcasecmp(mimestruct.content_type.value, "message")==0 &&
          stringcasecmp(mimestruct.content_type.subvalue, "rfc822")==0)
   {
    if(recursion<10)
     {
      rfcmessaget subpartm;

      subpartm.data=rfcm.data+mimestruct.offset+mimestruct.headerlen;
      subpartm.endofmail=subpartm.endoffile=subpartm.data+mimestruct.len;

      mimestructt substruct;
      substruct.mimeparse(subpartm);

      return dosearch_body_match(mimestruct, pattern, recursion+1);
     }
   }
  else if(mimestruct.sub.size()!=0)
   {
    if(recursion<10)
      for(unsigned i=0; i<mimestruct.sub.size(); i++)
        if(dosearch_body_match(mimestruct.sub[i], pattern, recursion+1))
          return TRUE;
   }    

  return FALSE;
 }

bool dboximap4t::dosearch_body_match(dword nr, const std::string &pattern)
 {
  if(get_structure(nr-1))
    return FALSE;

  return dosearch_body_match(*mimestruct, pattern, 0);
 }

bool dboximap4t::dosearch_text_match(dword nr, const std::string &pattern)
 {
  get_body(nr-1);

   {
    std::string header_text;
    header_text.assign(rfcm.data, rfcm.headerlen);
    if(dosearch_pattern_match(header_text, pattern))
      return TRUE;
   }

  return dosearch_body_match(nr, pattern);
 }

bool dboximap4t::dosearch(dword nr, unsigned int &argnr)
 {
  if(argnr>=arguments.size()) return FALSE;

  if(arguments[argnr].begin_list)
   {
    bool result=TRUE;

    for(argnr++; argnr<arguments.size() && 
        !arguments[argnr].end_list; argnr++)
      if(!dosearch(nr, argnr)) result=FALSE;

    return result;
   }

  const char *key=arguments[argnr++].data.c_str();

  if(isdigit(key[0]) || key[0]=='*') // message set
   {
    return dosearch_inset(key, nr);
   }
  else if(stricmp(key, "ALL")==0)
    return TRUE;
  else // requires index entry
   {
    get_folderindex(nr-1);

    if(stricmp(key, "ANSWERED")==0)
     {
      return (fi.flags.imapflags&DBOXIMAP4_FLAG_Answered)!=0;
     }
    else if(stricmp(key, "BCC")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_header_match(nr, "bcc", arguments[argnr++].data);
     }
    else if(stricmp(key, "BEFORE")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return fi.received<parse_date(arguments[argnr++].data);
     }
    else if(stricmp(key, "BODY")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_body_match(nr, arguments[argnr++].data);
     }
    else if(stricmp(key, "CC")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_header_match(nr, "cc", arguments[argnr++].data);
     }
    else if(stricmp(key, "DELETED")==0)
      return fi.flags.deleted;
    else if(stricmp(key, "DRAFT")==0)
      return (fi.flags.imapflags&DBOXIMAP4_FLAG_Draft)!=0;
    else if(stricmp(key, "FLAGGED")==0)
      return (fi.flags.imapflags&DBOXIMAP4_FLAG_Flagged)!=0;
    else if(stricmp(key, "FROM")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_header_match(nr, "from", arguments[argnr++].data);
     }
    else if(stricmp(key, "HEADER")==0)
     {
      if((argnr+1)>=arguments.size()) return FALSE;
      std::string header=arguments[argnr++].data;
      std::string pattern=arguments[argnr++].data;
      return dosearch_header_match(nr, header, pattern);
     }
    else if(stricmp(key, "KEYWORD")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      argnr++;
      return FALSE; // TODO
     }
    else if(stricmp(key, "LARGER")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return (fi.headerlen+fi.len)>
             (unsigned)atol(arguments[argnr++].data.c_str());
     }
    else if(stricmp(key, "NEW")==0)
     {
      return FALSE; // TODO
     }
    else if(stricmp(key, "NOT")==0)
     {
      return !dosearch(nr, argnr);
     }
    else if(stricmp(key, "OLD")==0)
     {
      return FALSE; // TODO
     }
    else if(stricmp(key, "ON")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_check_on(fi.received, parse_date(arguments[argnr++].data));
     }
    else if(stricmp(key, "OR")==0)
     {
      bool result1=dosearch(nr, argnr);
      bool result2=dosearch(nr, argnr);
      return result1 || result2;
     }
    else if(stricmp(key, "RECENT")==0)
     {
      return FALSE; // TODO
     }
    else if(stricmp(key, "SEEN")==0)
      return fi.flags.seen;
    else if(stricmp(key, "SENTBEFORE")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return fi.eda<parse_date(arguments[argnr++].data);
     }
    else if(stricmp(key, "SENTON")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_check_on(fi.received, parse_date(arguments[argnr++].data));
     }
    else if(stricmp(key, "SENTSINCE")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return fi.eda>=parse_date(arguments[argnr++].data);
     }
    else if(stricmp(key, "SINCE")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return fi.received>=parse_date(arguments[argnr++].data);
     }
    else if(stricmp(key, "SMALLER")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return (fi.headerlen+fi.len)<
             (unsigned)atol(arguments[argnr++].data.c_str());
     }
    else if(stricmp(key, "SUBJECT")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_header_match(nr, "subject", arguments[argnr++].data);
     }
    else if(stricmp(key, "TEXT")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_text_match(nr, arguments[argnr++].data);
     }
    else if(stricmp(key, "TO")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_header_match(nr, "to", arguments[argnr++].data);
     }
    else if(stricmp(key, "UID")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      return dosearch_inset(arguments[argnr++].data.c_str(), fi.id);
     }
    else if(stricmp(key, "UNANSWERED")==0)
      return (fi.flags.imapflags&DBOXIMAP4_FLAG_Answered)==0;
    else if(stricmp(key, "UNDELETED")==0)
      return !fi.flags.deleted;
    else if(stricmp(key, "UNDRAFT")==0)
      return (fi.flags.imapflags&DBOXIMAP4_FLAG_Draft)==0;
    else if(stricmp(key, "UNFLAGGED")==0)
      return (fi.flags.imapflags&DBOXIMAP4_FLAG_Flagged)==0;
    else if(stricmp(key, "UNKEYWORD")==0)
     {
      if(argnr>=arguments.size()) return FALSE;
      argnr++;
      return FALSE; // TODO
     }
    else if(stricmp(key, "UNSEEN")==0)
      return (fi.flags.imapflags&DBOXIMAP4_FLAG_Flagged)==0;
    else
      return FALSE;
   }
 }

void dboximap4t::dosearch(bool uid)
 {
  cout << "* SEARCH";

  for(dword nr=1; nr<=messages; nr++)
   {
    unsigned argnr=uid?1:0;
    
    if(argnr>=arguments.size()) break;

    if(stringcasecmp(arguments[argnr].data, "CHARSET")==0)
     {
      argnr++;
      if(argnr>=arguments.size()) break;
      argnr++;
      if(argnr>=arguments.size()) break;
     }

    bool result=TRUE;

    while(argnr<arguments.size())
      if(!dosearch(nr, argnr)) result=FALSE;

    if(result)
      cout << ' ' << (uid?fi.id:nr);
   }

  cout << "\r\n";
 }

