/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: IMAP4 STATUS command

*/

#ifndef WIN32
#include <unistd.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <iostream>

#include <stringtools.h>
#include "dboximap4.h"

void dboximap4t::dostatus()
 {
  if(arguments.size()<1)
   {
    std::cout << tag << " NO give a mailbox name\r\n";
    command_ok=FALSE;
    return; 
   }

  std::string mailbox_name=arguments[0].data, folder_owner, subfolder, folder_name;

  if(process_mailbox_name(mailbox_name, folder_owner, subfolder, folder_name))
    return;

  folder_grantst grants;
  ::get_folder_grants(folder_owner, subfolder, up, gp, grants);
  if(!grants.read)
   {
    std::cout << tag << " NO permission denied\r\n";
    command_ok=FALSE;
    return;
   }

  folderpt new_fp;
  new_fp.setname(folder_name);
  new_fp.read();

  std::string indexfilename=new_fp.getpath();
  indexfilename+="index";

  int index=open(indexfilename.c_str(), O_BINARY|O_RDONLY);
  dword messages_in_folder=0, recent=0, unseen=0;

  if(index>=0)
   {
    while(read(index, &fi, sizeof fi)==sizeof fi)
     {
      messages_in_folder++;
      if(!fi.flags.seen) unseen++;
     }

    ::close(index);
   }

  std::cout << "* STATUS ";
  send_string(arguments[0].data);
  std::cout << ' ';

  begin_list();

  for(unsigned argnr=1; argnr<arguments.size(); argnr++)
   {
    if(stringcasecmp(arguments[argnr].data, "MESSAGES")==0)
     {
      list_item(); std::cout << "MESSAGES";
      list_item(); std::cout << messages_in_folder;
     }
    else if(stringcasecmp(arguments[argnr].data, "RECENT")==0)
     {
      list_item(); std::cout << "RECENT";
      list_item(); std::cout << recent;
     }
    else if(stringcasecmp(arguments[argnr].data, "UIDNEXT")==0)
     {
      dword nextid=atol(new_fp|"nextid");
      if(nextid==0) nextid=1;
      new_fp.read();
      list_item(); std::cout << "UIDNEXT";
      list_item(); std::cout << nextid;
     }
    else if(stringcasecmp(arguments[argnr].data, "UIDVALIDITY")==0)
     {
      dword uidvalidity=atol(new_fp|"imap.uidvalidity");
      if(uidvalidity==0) uidvalidity=1;
      list_item(); std::cout << "UIDVALIDITY";
      list_item(); std::cout << uidvalidity;
     }
    else if(stringcasecmp(arguments[argnr].data, "UNSEEN")==0)
     {
      list_item(); std::cout << "UNSEEN";
      list_item(); std::cout << unseen;
     }
   }

  end_list();

  std::cout << "\r\n";
 }
